.view .button {
  background-color: shade(@theme_selected_bg_color, 1.1);
  background-image: none;
  border: none;
  color: rgba(255,255,255,0.45);
  border-radius: 0px; 
  margin: 0; 
}

.view .button:hover {
    background-color: shade(@theme_selected_bg_color, 1.45);
    transition: 200ms linear
}

.view .toolbar {
  background-color: @theme_selected_bg_color;  
  border-radius: 0px;
}

.view .toolbarEnd {
  border-left: rgba(255,255,255,0.45) 1px solid; 
}

.view .toolbarEnd:hover {
  border-left: rgba(255,255,255,0.45) 1px solid;
  transition: 200ms linear; 
}

.view .toolbarSecond {
  border-right: rgba(255,255,255,0.45) 1px solid;
}

.view .toolbarSecond:hover {
  border-right: rgba(255,255,255,0.45) 1px solid;
  transition: 200ms linear; 
}

.emptyGrid {
   background-color: @theme_bg_color;
}

.list-row {
  -GtkWidget-focus-padding: 0;
  -GtkWidget-focus-line-width: 0;
}
